<?php
require_once 'PHPExcel.php';

/*******************************
* GENERAL NOTES:
* --------------
* row index is 1-based 
* col index is 0-based
* therefore A1 is 0,1
********************************/


// ****************************************
//  Start of meta data functions
// ****************************************
function excel_GetSheetNames($excelFile,$excelFormat){
	// returns the sheet names in an array
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(True);
	$objPHPExcel=$objReader->load($excelFile);
	return $objWorksheet=$objPHPExcel->getSheetNames();
}

function excel_GetSheetActiveIndex($excelFile,$excelFormat){
	// returns the number for the active sheet
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(True);
	$objPHPExcel=$objReader->load($excelFile);
	return $objWorksheet=$objPHPExcel->getActiveSheetIndex();
}

function excel_GetSheetCount($excelFile,$excelFormat){
	// returns the number of sheets found in the excel workbook
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(True);
	$objPHPExcel=$objReader->load($excelFile);
	return $objWorksheet=$objPHPExcel->getSheetCount();
}

function excel_getRowCount($excelFile,$excelFormat,$excelSheetNum){
	// returns the number of rows found in the sheet specified
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(True);
	$objPHPExcel=$objReader->load($excelFile);
	$objWorksheet=$objPHPExcel->getSheet($excelSheetNum);
	return $objWorksheet->getHighestRow(); // e.g. 10
}

function excel_getColumnCount($excelFile,$excelFormat,$excelSheetNum){
	// returns the number of Columns found in the sheet specified
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(True);
	$objPHPExcel=$objReader->load($excelFile);
	$objWorksheet=$objPHPExcel->getSheet($excelSheetNum);
	$highestColumn=$objWorksheet->getHighestColumn();	// e.g 'F'
	return PHPExcel_Cell::columnIndexFromString($highestColumn);	// e.g. 5
}

function excel_GetStyleFromCell($excelFile,$excelFormat,$excelSheetNum,$row,$col){
	// this returns style info (formatcode) from a specific cell (row/col) of the sheet
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(false);
	$objPHPExcel=$objReader->load($excelFile);
	$objWorksheet=$objPHPExcel->getSheet($excelSheetNum);
	$cellID = $objWorksheet ->getCellByColumnAndRow($col, $row)->getCoordinate();
	$cell = $objWorksheet->getCell($cellID);
	return $objPHPExcel->getCellXfByIndex( $cell->getXfIndex() )->getNumberFormat()->getFormatCode();
}

// ****************************************
//  Start of 'getData' functions
// ****************************************
function excel_GetDataFromCell($excelFile,$excelFormat,$excelSheetNum,$row,$col){
	// this returns data from a specific cell (row/col) of the sheet
	// USE WITH CAUTION :: VERY SLOW TO RECURSE THROUGH AN ENTIRE WORKBOOK
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(True);
	$objPHPExcel=$objReader->load($excelFile);
	$objWorksheet=$objPHPExcel->getSheet($excelSheetNum);
	// return $objWorksheet->getCellByColumnAndRow($col,$row)->getValue();
	// new code below
	$cellID = $objWorksheet ->getCellByColumnAndRow($col, $row)->getCoordinate();
	$cell = $objWorksheet->getCell($cellID);
	$FormatMask = $objPHPExcel->getCellXfByIndex( $cell->getXfIndex() )->getNumberFormat()->getFormatCode();
	if(is_null($cell->getCalculatedValue())){
		return "";	
	}else{
		return PHPExcel_Style_NumberFormat::toFormattedString( $cell->getCalculatedValue(), $FormatMask);
	}
	
}

function excel_GetDataFromRow($excelFile,$excelFormat,$excelSheetNum,$row){
	// gets data from an entire row and returns it as an array
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(False);
	$objPHPExcel=$objReader->load($excelFile);
	$objWorksheet=$objPHPExcel->getSheet($excelSheetNum);
	$highestColumn=$objWorksheet->getHighestColumn();	// e.g 'F'
	$highestColumnIndex=PHPExcel_Cell::columnIndexFromString($highestColumn);	// e.g. 5
	$myArray = array();
	For ($col=0;$col<=$highestColumnIndex;++$col){
		$cellID = $objWorksheet ->getCellByColumnAndRow($col, $row)->getCoordinate();
		$cell = $objWorksheet->getCell($cellID);
		$FormatMask = $objPHPExcel->getCellXfByIndex( $cell->getXfIndex() )->getNumberFormat()->getFormatCode();
		if(is_null($cell->getCalculatedValue())){
			$myArray[$col] = "";	
		}else{
			$myArray[$col] = PHPExcel_Style_NumberFormat::toFormattedString( $cell->getCalculatedValue(), $FormatMask);
		}
	}
	return $myArray;
}

function excel_GetDataFromColumn($excelFile,$excelFormat,$excelSheetNum,$col){
	// gets data from an entire column and returns it as an array
	$objReader=PHPExcel_IOFactory::createReader($excelFormat);
	$objReader->setReadDataOnly(False);
	$objPHPExcel=$objReader->load($excelFile);
	$objWorksheet=$objPHPExcel->getSheet($excelSheetNum);
	$highestRow=$objWorksheet->getHighestRow();	// e.g '50'
	$myArray = array();
	For ($row=1;$row<=$highestRow;++$row){
		$cellID = $objWorksheet ->getCellByColumnAndRow($col, $row)->getCoordinate();
		$cell = $objWorksheet->getCell($cellID);
		$FormatMask = $objPHPExcel->getCellXfByIndex( $cell->getXfIndex() )->getNumberFormat()->getFormatCode();
		if(is_null($cell->getCalculatedValue())){
			// because the row starts at 1 and not 0, offset the array elements by 1 
			// to get the proper array structure in 4D
			$myArray[$row-1] = "";	
		}else{
			// because the row starts at 1 and not 0, offset the array elements by 1 
			// to get the proper array structure in 4D
			$myArray[$row-1] = PHPExcel_Style_NumberFormat::toFormattedString( $cell->getCalculatedValue(), $FormatMask);
		}
	}
	return $myArray;
}

?>

